/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import datatypes.DuplicateGroupException;
import datatypes.DuplicateMessageException;
import datatypes.GroupIterator;
import datatypes.GroupMap;
import datatypes.MessageIDIterator;
import datatypes.MessageMap;
import datatypes.NewsItem;
import datatypes.NewsItemIterator;
import datatypes.NoSuchGroupException;
import datatypes.NoSuchMessageException;

/**
 * An implementation of the GroupMap interfaces for managing news groups.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.1, 03/28/03
 */
public class BasicGroupMap implements GroupMap {
    private Map groups;

    /**
     * Creates a new BasicGroupMap object.
     */
    public BasicGroupMap() {
        this.groups = new Hashtable();
    }

    /**
     * Queries if this container has a group with a particular name.
     *
     * @param groupName The name of the group for which to check.
     *
     * @return true if this group already exists; false otherwise.
     */
    public boolean containsGroup(String groupName) {
        return this.groups.containsKey(groupName);
    }

    /**
     * Creates a new group.
     *
     * @param groupName The name of the group to create.
     *
     * @throws DuplicateGroupException if there is already a group by this
     *         name.
     */
    public void addGroup(String groupName) throws DuplicateGroupException {
        // check for duplicate groups and disallow
        if (this.containsGroup(groupName)) {
            throw new DuplicateGroupException();
        }

        // build the message map and store it in the group map
        MessageMap messages = new BasicMessageMap();
        this.groups.put(groupName, messages);
    }

    /**
     * Gets all the group names in this container.
     *
     * @return A GroupIterator of all the group names.
     */
    public GroupIterator getGroups() {
        Iterator iterator = this.groups.keySet().iterator();

        return new BasicGroupIterator(iterator);
    }

    /**
     * Gets all the message IDs for a particular group.
     *
     * @param groupName The name of the group to retrieve message IDs for.
     *
     * @return A MessageIDIterator containing all the message IDs.
     *
     * @throws NoSuchGroupException if no such group exists.
     */
    public MessageIDIterator getMessages(String groupName)
                                  throws NoSuchGroupException {
        if (!this.containsGroup(groupName)) {
            throw new NoSuchGroupException();
        }

        // get the message map
        MessageMap messages = (MessageMap)this.groups.get(groupName);

        // return the message ID iterator
        return messages.getMessages();
    }

    /**
     * Posts a message to one of the groups.
     *
     * @param message The NewsItem to post.
     *
     * @throws DuplicateMessageException if this message already exists.
     * @throws NoSuchGroupException if this message is for a non-existant
     *         group.
     */
    public void postMessage(NewsItem message)
                     throws DuplicateMessageException, NoSuchGroupException {
        String groupName = message.getGroup();

        // confirm the posting group exists...
        if (!this.containsGroup(groupName)) {
            throw new NoSuchGroupException();
        }

        // get the message map and add the new message to it
        MessageMap messages = (MessageMap)this.groups.get(groupName);
        messages.addMessage(message);
    }

    /**
     * Queries if this container has a message with a particular message ID.
     *
     * @param messageID The ID of the message to search for.
     *
     * @return true if the message is in this container; false otherwise.
     */
    public boolean containsMessage(String messageID) {
        try {
            this.getMessage(messageID);
        } catch (NoSuchMessageException e) {
            return false;
        }

        return true;
    }

    /**
     * Gets the message associated with a message ID.
     *
     * @param messageID The ID of the message to retrieve.
     *
     * @return The retrieved message.
     *
     * @throws NoSuchMessageException if no message exists with this message
     *         ID.
     */
    public NewsItem getMessage(String messageID)
                        throws NoSuchMessageException {
        Iterator groups = this.groups.values().iterator();

        while (groups.hasNext()) {
            MessageMap messages = (MessageMap)groups.next();

            if (messages.containsMessage(messageID)) {
                return messages.getMessage(messageID);
            }
        }

        throw new NoSuchMessageException();
    }

    /**
     * Gets the header of a message with a particular message ID.
     *
     * @param messageID The ID of the message for which to find the header.
     *
     * @return The message header.
     *
     * @throws NoSuchMessageException if no message exists with this message
     *         ID.
     */
    public String getHeader(String messageID) throws NoSuchMessageException {
        NewsItem message = this.getMessage(messageID);

        return message.getHeader();
    }

    /**
     * Gets all the messages since a particular date.
     *
     * @param sinceWhen The date to find messages since.
     *
     * @return A NewsItemIterator containing all the messages posted since
     *         the date.
     */
    public NewsItemIterator getMessages(Date sinceWhen) {
        List list = new ArrayList();
        Iterator groups = this.groups.values().iterator();

        while (groups.hasNext()) {
            MessageMap group = (MessageMap)groups.next();

            list.addAll(group.getMessages(sinceWhen));
        }

        return new SimpleNewsItemIterator(list.iterator());
    }

    /**
     * An implementation of the NewItemIterator interface for iterating over
     * a set of NewsItems.
     *
     * @author John David Ratliff
     * @author Yohei Kaneko
     * @version 1.1, 03/28/03
     */
    private class SimpleNewsItemIterator implements NewsItemIterator {
        private Iterator iterator;

        /**
         * Creates a new SimpleNewsItemIterator object.
         *
         * @param iterator The iterator containing all the NewsItems.
         */
        protected SimpleNewsItemIterator(Iterator iterator) {
            this.iterator = iterator;
        }

        /**
         * Queries if there are more news items in this iterator.
         *
         * @return true if more news items exist; false otherwise.
         */
        public boolean hasMoreNewsItems() {
            return this.iterator.hasNext();
        }

        /**
         * Retrieves the next NewsItem in this iterator.
         *
         * @return The next NewsItem.
         */
        public NewsItem nextNewsItem() {
            return (NewsItem)this.iterator.next();
        }
    }
}
