/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import java.util.Date;

import datatypes.NewsItem;
import datatypes.XMLFactory;

import org.gjt.xpp.XmlNode;

/**
 * An implemtation of the NewsItem interface for storing news items and
 * doing conversions to and from XML.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.6, 04/11/03
 */
public class BasicNewsItem implements NewsItem {
    private String group;
    private String messageID;
    private String from;
    private String header;
    private String body;
    private Date date;

    /**
     * Creates a new BasicNewsItem object.
     *
     * @param group The news group this message is for.
     * @param messageID The ID of this message.
     * @param from The poster of this message (e.g. who this message is
     *        from).
     * @param header The message header.
     * @param body The message body.
     * @param date The posting date of this message.
     */
    public BasicNewsItem(String group, String messageID, String from,
                         String header, String body, Date date) {
        this.group = group;
        this.messageID = messageID;
        this.header = header;
        this.from = from;
        this.body = body;
        this.date = date;
    }

    /**
     * Gets the newsgroup this news item is for.
     *
     * @return The newsgroup.
     */
    public String getGroup() {
        return this.group;
    }

    /**
     * Gets the message ID of this news item.
     *
     * @return The message ID.
     */
    public String getMessageID() {
        return this.messageID;
    }

    /**
     * Gets the poster of this message.
     *
     * @return The poster of this message.
     */
    public String getFrom() {
        return this.from;
    }

    /**
     * Gets the header of this message.
     *
     * @return The message header.
     */
    public String getHeader() {
        return this.header;
    }

    /**
     * Gets the body of this message.
     *
     * @return The message body.
     */
    public String getBody() {
        return this.body;
    }

    /**
     * Gets the date this message was posted.
     *
     * @return The message's post date.
     */
    public Date getDate() {
        return this.date;
    }
    
    /**
     * Sets the date of this message.
     *
     * @param date The new date of this message.
     */
    public void setDate(Date date) {
        this.date = date;
    }

    /**
     * Creates an XmlNode representation of this object. Based on code
     * provided by Professor Dennis Gannon.
     *
     * @return The XmlNode representation.
     */
    public XmlNode toXmlNode() {
        // build the data tags
        String group = "<group>" + this.group + "</group>";
        String messageID = "<messageID>" + this.messageID + "</messageID>";
        String from = "<from>" + this.from + "</from>";
        String header = "<header>" + this.header + "</header>";
        String body = "<body>" + this.body + "</body>";
        String date = "<date>" + this.date.getTime() + "</date>";

        // build the object tag
        String xmlString = "<news-item>" + group + messageID + from +
                           header + body + date + "</news-item>";

        // get the factory
        XMLFactory factory = BasicXMLFactory.getInstance();

        // ask the factory to create the XmlNode
        return factory.buildXmlNode(xmlString);
    }

    /**
     * Queries if two BasicNewsItems are equivalent. Returns true only if
     * all the news fields are equivalent (e.g. body, date, from, etc.).
     *
     * @param object The object to test equivalence with.
     *
     * @return true for equal objects; false otherwise.
     */
    public boolean equals(Object object) {
        try {
            BasicNewsItem item = (BasicNewsItem)object;

            if (!this.body.equals(item.getBody())) {
                return false;
            }

            if (!this.date.equals(item.getDate())) {
                return false;
            }

            if (!this.from.equals(item.getFrom())) {
                return false;
            }

            if (!this.group.equals(item.getGroup())) {
                return false;
            }

            if (!this.header.equals(item.getHeader())) {
                return false;
            }

            if (!this.messageID.equals(item.getMessageID())) {
                return false;
            }
        } catch (ClassCastException e) {
            // the object was not a BasicNewsItem
            return false;
        }

        return true;
    }

    /**
     * Returns a string representation of a BasicNewsItem.
     *
     * @return The string representation.
     */
    public String toString() {
        return "BasicNewsItem[group = " + this.group +
               "\n              messageID = " + this.messageID +
               "\n              from = " + this.from +
               "\n              header = " + this.header +
               "\n              date = " + this.date + "]";
    }
}
