/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import java.io.IOException;
import java.io.StringReader;

import datatypes.XMLFactory;

import org.gjt.xpp.XmlNode;
import org.gjt.xpp.XmlPullParser;
import org.gjt.xpp.XmlPullParserException;
import org.gjt.xpp.XmlPullParserFactory;

/**
 * An implementation of the XMLFactory interface for creating complete XML
 * objects from XML substrings.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.2, 04/01/03
 */
public class BasicXMLFactory implements XMLFactory {
    private XmlPullParserFactory xmlFactory;
    private static final XMLFactory INSTANCE = new BasicXMLFactory();

    /**
     * Creates a new BasicXMLFactory object. Outside creation is disallowed,
     * so only one factory is ever built.
     */
    private BasicXMLFactory() {
        // let only one factory be built.
        try {
            this.xmlFactory = XmlPullParserFactory.newInstance();
        } catch (XmlPullParserException e) {
            // this is a huge error
            System.err.println("BasicXMLFactory: unable to build XmlPullParserFactory!");
        }
    }

    /**
     * Gets a reference to the factory.
     *
     * @return An XMLFactory reference to this factory.
     */
    public static XMLFactory getInstance() {
        return INSTANCE;
    }

    /**
     * Builds an XmlNode from an xml substring. Based on code provided by
     * Professor Dennis Gannon.
     *
     * @param xmlString The XML substring.
     *
     * @return The newly created XmlNode.
     */
    public XmlNode buildXmlNode(String xmlString) {
        XmlNode node = null;

        try {
            // build the XML parser
            XmlPullParser parser = this.xmlFactory.newPullParser();

            // setup the input
            parser.setInput(new StringReader(xmlString));
            parser.setNamespaceAware(true);
            parser.next();

            // build the XML tree
            node = this.xmlFactory.newNode(parser);
        } catch (XmlPullParserException e) {
            // this should rarely (if ever) happen
            e.printStackTrace();
        } catch (IOException e) {
            // this is a parse error. another Bad Thing (tm).
            System.err.println("BasicXMLFactory: parse error converting\n" +
                               xmlString);
        }

        return node;
    }
}
