/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import datatypes.DuplicateGroupException;
import datatypes.DuplicateMessageException;
import datatypes.GroupIterator;
import datatypes.MessageIDIterator;
import datatypes.NewsItemIterator;
import datatypes.NoSuchGroupException;
import datatypes.NoSuchMessageException;

import org.gjt.xpp.XmlNode;

import protocol.AbstractNntp;
import protocol.Nntp;
import protocol.ServerBindException;

import soaprmi.RemoteException;

/**
 * Unit Tests for the package datastructures.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.8, 04/11/03
 */
public class UnitTests {
    /**
     * Tests the datastructures.BasicNewsItem class.
     */
    public static void testBasicNewsItem() {
        // test method for the BasicNewsItem class
        String group = "ac.csci.b538";
        String messageID = "123@yohei.com";
        String from = "yohei@yohei.com";
        String header = "test message alpha";
        String body = "disregard this message, please.";
        Date date = new Date();

        BasicNewsItem message = new BasicNewsItem(group, messageID, from,
                                                  header, body, date);

        // confirm the get methods work
        assert (group.equals(message.getGroup()));
        assert (messageID.equals(message.getMessageID()));
        assert (from.equals(message.getFrom()));
        assert (header.equals(message.getHeader()));
        assert (body.equals(message.getBody()));
        assert (date.equals(message.getDate()));

        BasicNewsItem message2 = new BasicNewsItem(group, messageID, from,
                                                   header, body, date);

        // confirm equality
        assert (message.equals(message2));

        // confirm inequality
        message2 = new BasicNewsItem("", messageID, from, header, body, date);

        assert (!message.equals(message2));

        message2 = new BasicNewsItem(group, "", from, header, body, date);

        assert (!message2.equals(message));

        message2 = new BasicNewsItem(group, messageID, "", header, body, date);

        assert (!message.equals(message2));

        message2 = new BasicNewsItem(group, messageID, from, "", body, date);

        assert (!message2.equals(message));

        message2 = new BasicNewsItem(group, messageID, from, header, "", date);

        assert (!message.equals(message2));

        message2 = new BasicNewsItem(group, messageID, from, header, body,
                                     new Date());

        assert (!message2.equals(message));

        // test XML serialization (toXmlNode() and constructor(XmlNode))
        BasicNewsItem message3 = new BasicNewsItem("ac.csci.c311",
                                                   "199@localhost", "yohei",
                                                   "yohei's header",
                                                   "blah blah", new Date());

        XmlNode serialized = message3.toXmlNode();

        try {
            BasicNewsItem message4 = (BasicNewsItem)XMLParser.parseNewsItem(serialized);

            assert (message3.equals(message4));
            assert (!message4.equals(message2));
        } catch (XMLParseException e) {
            // this should NOT happen
            assert false;
        }

        // if we made it to this point, all is well
        System.out.println("testBasicNewsItem: all tests passed!");
    }

    /**
     * Tests the datastructures.BasicMessageMap class.
     */
    public static void testBasicMessageMap() {
        // the message map to test
        BasicMessageMap map = new BasicMessageMap();

        // some message variables to create messages with
        String group = "ac.csci.b438";
        String messageID = "123@yohei.com";
        String from = "yohei@yohei.com";
        String header = "test message alpha";
        String body = "disregard this message, please.";
        Date date = new Date(100);

        // a simple message item        
        BasicNewsItem message = new BasicNewsItem(group, messageID, from,
                                                  header, body, date);

        // test the addMessage() method
        map.addMessage(message);

        // exception DuplicateMessageException
        try {
            map.addMessage(message);

            // should never reach here
            assert false;
        } catch (DuplicateMessageException e) {
        }

        // test the containsMessage() method
        assert (map.containsMessage(message.getMessageID()));

        // does it not contain messages we have not added
        assert (!map.containsMessage(""));

        // test the getHeader() method
        assert (header.equals(map.getHeader(message.getMessageID())));

        // exception NoSuchMessageException
        try {
            map.getHeader("no such message ID");

            // this should never reach here
            assert false;
        } catch (NoSuchMessageException e) {
        }

        // test the getMessage() method
        assert (message.equals(map.getMessage(message.getMessageID())));

        // exception NoSuchMessageException
        try {
            map.getMessage("no such message ID");

            // this should never reach here
            assert false;
        } catch (NoSuchMessageException e) {
        }

        // create the next date
        Date date2 = new Date(200);

        BasicNewsItem message2 = new BasicNewsItem("ac.csci.b438",
                                                   "124@yohei.com",
                                                   "dom@yohei.com",
                                                   "some header", "yeah.",
                                                   date2);

        BasicNewsItem message3 = new BasicNewsItem("ac.csci.b438",
                                                   "125@yohei.com",
                                                   "yohei@yohei.com",
                                                   "new header", "iie.",
                                                   date2);

        BasicNewsItem message4 = new BasicNewsItem("ac.csci.b438",
                                                   "126@yohei.com",
                                                   "dom@yohei.com",
                                                   "linguistics major",
                                                   "almost.", date2);

        BasicNewsItem message5 = new BasicNewsItem("ac.csci.b438",
                                                   "127@yohei.com",
                                                   "dom@yohei.com",
                                                   "some header", "yeah.",
                                                   date2);

        map.addMessage(message2);
        map.addMessage(message3);
        map.addMessage(message4);
        map.addMessage(message5);

        MessageIDIterator iterator = map.getMessages();

        while (iterator.hasMoreMessageIDs()) {
            String id = iterator.nextMessageID();

            // the ID must match one of the messages we added
            assert ((id.equals(message.getMessageID())) ||
            (id.equals(message2.getMessageID())) ||
            (id.equals(message3.getMessageID())) ||
            (id.equals(message4.getMessageID())) ||
            (id.equals(message5.getMessageID())));
        }

        // include the first message
        Collection collection = map.getMessages(date);
        assert (collection.size() == 5);

        // don't include the first message
        collection = map.getMessages(date2);
        assert (collection.size() == 4);

        // don't include any messages
        Date date3 = new Date(date2.getTime() + 1);
        collection = map.getMessages(date3);
        assert (collection.size() == 0);

        System.out.println("testBasicMessageMap: all tests passed!");
    }

    /**
     * Tests the datastructures.BasicGroupMap class.
     */
    public static void testBasicGroupMap() {
        // build the group map to test
        BasicGroupMap groups = new BasicGroupMap();

        // build some groups to add to the map
        String group = "ac.csci.b538";
        String group2 = "ac.csci.c343";
        String group3 = "cs.system";
        String group4 = "ac.csci.c211";
        String group5 = "ac.csci.c212";

        // test containsGroup() method
        assert (!groups.containsGroup(group));

        groups.addGroup(group);
        assert (groups.containsGroup(group));
        assert (!groups.containsGroup(group2));

        // add the other groups
        groups.addGroup(group2);
        groups.addGroup(group3);
        groups.addGroup(group4);
        groups.addGroup(group5);

        try {
            // already contains this group
            // should throw DuplicateGroupException
            groups.addGroup(group3);

            // this should never get here
            assert false;
        } catch (DuplicateGroupException e) {
        }

        // test the getGroups() method        
        GroupIterator iterator = groups.getGroups();

        while (iterator.hasMoreGroups()) {
            String nextGroup = iterator.nextGroup();

            // must be one of these groups
            assert (nextGroup.equals(group) || nextGroup.equals(group2) ||
            nextGroup.equals(group3) || nextGroup.equals(group4) ||
            nextGroup.equals(group5));
        }

        // create dates for the messages
        Date date1 = new Date(100);
        Date date2 = new Date(200);

        // create messages to post
        BasicNewsItem message1 = new BasicNewsItem(group, "100@localhost",
                                                   "yohei", "first post!",
                                                   "yah!", date1);

        BasicNewsItem message2 = new BasicNewsItem(group, "101@localhost",
                                                   "dom", "second post!",
                                                   "woohoo!", date2);

        // test the postMessage() method
        groups.postMessage(message1);
        groups.postMessage(message2);

        // test that duplicate messages throw DuplicateMessageException
        try {
            // should throw exception
            groups.postMessage(message1);

            // will never reach here. error if it does
            assert false;
        } catch (DuplicateMessageException e) {
        }

        // test that posting messages to non-existant groups fails
        try {
            BasicNewsItem fail = new BasicNewsItem("ac.csci.nosuchgroup",
                                                   "", "", "", "", null);

            groups.postMessage(fail);

            // this should never get here
            assert false;
        } catch (NoSuchGroupException e) {
        }

        assert (groups.containsMessage(message1.getMessageID()));
        assert (groups.containsMessage(message2.getMessageID()));
        assert (!groups.containsMessage("some random message"));

        // test getMessages() method
        try {
            // no such group exception
            MessageIDIterator fail = groups.getMessages("ac.csci.nonexistant");

            // we should never get here...
            assert false;
        } catch (NoSuchGroupException e) {
        }

        MessageIDIterator empty = groups.getMessages(group2);

        assert (!empty.hasMoreMessageIDs());

        MessageIDIterator messages = groups.getMessages(group);

        while (messages.hasMoreMessageIDs()) {
            // it must be either message1 or message2
            String id = messages.nextMessageID();

            assert (id.equals(message1.getMessageID()) ||
            id.equals(message2.getMessageID()));
        }

        // test getMessages(Date) method
        Date date3 = new Date(date2.getTime() + 100);

        BasicNewsItem message3 = new BasicNewsItem(group3, "102@localhost",
                                                   "yohei@localhost",
                                                   "what's a header",
                                                   "some body", date3);

        groups.postMessage(message3);

        NewsItemIterator since = groups.getMessages(new Date(date1.getTime() +
                                                             10));

        while (since.hasMoreNewsItems()) {
            BasicNewsItem msg = (BasicNewsItem)since.nextNewsItem();

            assert (msg.equals(message2) || msg.equals(message3));
        }

        // test getHeader() method
        assert (groups.getHeader(message1.getMessageID()).equals(message1.getHeader()));
        assert (!groups.getHeader(message2.getMessageID()).equals(""));

        try {
            // throws a no such message exception
            groups.getHeader("no such message ID");

            // this should never get here
            assert false;
        } catch (NoSuchMessageException e) {
        }

        // test getMessage() method
        assert (groups.getMessage(message1.getMessageID()).equals(message1));
        assert (groups.getMessage(message3.getMessageID()).equals(message3));
        assert (!groups.getMessage(message2.getMessageID()).equals(""));

        try {
            // throws a no such message exception
            groups.getMessage("no such message ID");

            // this should never get here
            assert false;
        } catch (NoSuchMessageException e) {
        }

        System.out.println("testBasicGroupMap: all tests passed!");
    }

    /**
     * Tests the datastructures.BasicGroupIterator class.
     */
    public static void testBasicGroupIterator() {
        // create groups to test the iterator with
        String group = "ac.csci.c343";
        String group2 = "ac.csci.c311";
        String group3 = "ac.csci.b538";

        // build an ArrayList and use it's iterator to test
        List list = new ArrayList();

        list.add(group);
        list.add(group2);
        list.add(group3);

        GroupIterator iterator = new BasicGroupIterator(list.iterator());

        // make sure our elements are there
        // we must have some groups, we just added them
        assert (iterator.hasMoreGroups());

        // for any groups in the iterator, they should be the ones
        // we added...
        while (iterator.hasMoreGroups()) {
            String nextGroup = iterator.nextGroup();

            assert (nextGroup.equals(group) || nextGroup.equals(group2) ||
            nextGroup.equals(group3));
        }

        System.out.println("testBasicGroupIterator: all tests passed!");
    }

    /**
     * Tests the datastructures.XMLParser class.
     */
    public static void testXMLParser() {
        String messageID = "101.yohei@localhost";
        String messageID2 = "102.dom@localhost";
        String messageID3 = "103.gannon@whale.cs.indiana.edu";

        // create a message to convert and then reinstantiate
        BasicNewsItem message = new BasicNewsItem("ac.csci.c343", messageID,
                                                  "notyou", "me!", "body",
                                                  new Date());
        BasicNewsItem message2 = new BasicNewsItem("ac.csci.c343",
                                                   messageID2, "notyou",
                                                   "me!", "body", new Date());
        BasicNewsItem message3 = new BasicNewsItem("ac.csci.b538",
                                                   messageID3, "notyou",
                                                   "me!", "body", new Date());

        // convert the message to XML
        XmlNode xmlNewsItem = message.toXmlNode();

        try {
            // try to rebuild it
            BasicNewsItem rebuilt = (BasicNewsItem)XMLParser.parseNewsItem(xmlNewsItem);

            // they should be equal
            assert (rebuilt.equals(message));
        } catch (XMLParseException e) {
            assert false;
        }

        try {
            Nntp server = AbstractNntp.getInstance();
            XmlNode xmlGroupList = server.list();

            // this test will fail if the server's seed groups changes
            // no good way around this, I'm afraid...
            String group = "ac.csci.b538";
            String group2 = "ac.csci.c343";
            String group3 = "ac.csci.a348";
            String nextGroup;

            GroupIterator iterator = XMLParser.parseGroupList(xmlGroupList);

            // the iterator MUST have something
            assert (iterator.hasMoreGroups());

            nextGroup = iterator.nextGroup();

            assert (nextGroup.equals(group) || nextGroup.equals(group2) ||
            nextGroup.equals(group3));

            assert (iterator.hasMoreGroups());

            nextGroup = iterator.nextGroup();

            assert (nextGroup.equals(group) || nextGroup.equals(group2) ||
            nextGroup.equals(group3));

            assert (iterator.hasMoreGroups());

            nextGroup = iterator.nextGroup();

            assert (nextGroup.equals(group) || nextGroup.equals(group2) ||
            nextGroup.equals(group3));

            // only three groups...
            assert (!iterator.hasMoreGroups());

            // post the fake messages to the server
            server.iHave(message.toXmlNode());
            server.iHave(message2.toXmlNode());
            server.iHave(message3.toXmlNode());

            // check for any new messages since the epoch
            String nextID;
            XmlNode xmlNewNews = server.newNews(0);
            MessageIDIterator iterator2 = XMLParser.parseNewNews(xmlNewNews);

            assert (iterator2.hasMoreMessageIDs());

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2) ||
            nextID.equals(messageID3));

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2) ||
            nextID.equals(messageID3));

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2) ||
            nextID.equals(messageID3));

            assert (!iterator2.hasMoreMessageIDs());

            // check for news in group "ac.csci.c343"
            // expecting 2 messages (message and message2)
            XmlNode xmlGroupIDs = server.groupIDs("ac.csci.c343");
            iterator2 = XMLParser.parseGroupIDs(xmlGroupIDs);

            assert (iterator2.hasMoreMessageIDs());

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2));

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2));

            assert (!iterator2.hasMoreMessageIDs());
        } catch (XMLParseException e) {
            // we built an invalid list. this should not happen
            assert false;
        } catch (RemoteException e) {
            // this cannot happen. the server's not remote...
            assert false;
        } catch (ServerBindException e) {
            // wasn't able to create the server. this is unlikely to happen
            assert false;
        }

        System.out.println("testXMLParser: all tests passed!");
    }

    /**
     * Tests the datastructures.BasicGroupIterator class.
     */
    public static void testBasicMessageIDIterator() {
        // create message IDs to test the iterator with
        String id = "101.yohei@localhost";
        String id2 = "102.dom@localhost";
        String id3 = "103.gannon@whale.cs";

        // build an ArrayList and use it's iterator to test
        List list = new ArrayList();

        list.add(id);
        list.add(id2);
        list.add(id3);

        MessageIDIterator iterator = new BasicMessageIDIterator(list.iterator());

        // make sure our elements are there
        // we must have some groups, we just added them
        assert (iterator.hasMoreMessageIDs());

        // for any groups in the iterator, they should be the ones
        // we added...
        while (iterator.hasMoreMessageIDs()) {
            String nextID = iterator.nextMessageID();

            assert (nextID.equals(id) || nextID.equals(id2) ||
            nextID.equals(id3));
        }

        System.out.println("testBasicMessageIDIterator: all tests passed!");
    }

    /**
     * Runs the unit tests for the package datastructures. You must enable
     * compile-time assertion support in the Java 1.4 compiler and run-time
     * assertion enabling in the java runtime. To do this, use javac
     * -source 1.4 datastructures.UnitTests to compile and java -ea
     * datastructures.UnitTests to run.
     *
     * @param args Not used.
     */
    public static void main(String[] args) {
        // test the BasicNewsItem class
        testBasicNewsItem();

        // test the BasicMessageMap class
        testBasicMessageMap();

        // test the BasicGroupMap class
        testBasicGroupMap();

        // test the BasicGroupIterator class
        testBasicGroupIterator();

        // test the XMLParser class
        testXMLParser();

        // test the BasicMessageIDIterator class
        testBasicMessageIDIterator();

        // BasicXMLFactory, UnitTests, and XMLParserException need no tests
        // 
        // BasicXMLFactory only uses another codebase to do it's work
        //                 therefore, any problems are the problem of that
        //                 codebase, not ours
        // UnitTests - this one is self-explanatory
        // XMLParserException - it is just an extended Exception class
    }
}
