/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;

import datatypes.GroupIterator;
import datatypes.MessageIDIterator;
import datatypes.NewsItem;

import org.gjt.xpp.XmlNode;

/**
 * A collection of methods for parsing XML documents back into usable object
 * formats.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.4, 04/04/03
 */
public class XMLParser {
    /**
     * Creates a new XMLTranslator object.
     */
    private XMLParser() {
        // this class is for static factory method
        // disallow foreign creation
    }

    /**
     * Parses an XmlNode into a NewsItem object. Based on code provided by
     * Professor Dennis Gannon.
     *
     * @param xmlNewsItem The XmlNode representation to build from.
     *
     * @return The new NewsItem object.
     *
     * @throws XMLParseException if the xmlNewsItem document does not
     *         represent a valid NewsItem.
     */
    public static NewsItem parseNewsItem(XmlNode xmlNewsItem)
                                  throws XMLParseException {
        String group = null;
        String messageID = null;
        String from = null;
        String header = null;
        String body = null;
        Date date = null;

        if (xmlNewsItem.getLocalName().equals("news-item")) {
            // confirm we are a news-item object
            Enumeration elements = xmlNewsItem.children();

            // now get all the NewsItem properties
            while (elements.hasMoreElements()) {
                XmlNode element = (XmlNode)elements.nextElement();

                String name = element.getLocalName();

                if (name.equals("group")) {
                    group = (String)element.getChildAt(0);
                } else if (name.equals("messageID")) {
                    messageID = (String)element.getChildAt(0);
                } else if (name.equals("from")) {
                    from = (String)element.getChildAt(0);
                } else if (name.equals("header")) {
                    header = (String)element.getChildAt(0);
                } else if (name.equals("body")) {
                    body = (String)element.getChildAt(0);
                } else if (name.equals("date")) {
                    String milliseconds = (String)element.getChildAt(0);

                    try {
                        long time = Long.parseLong(milliseconds);
                        date = new Date(time);
                    } catch (NumberFormatException e) {
                        // invalid long
                        System.err.println("BasicNewsItem: invalid date string " +
                                           milliseconds);
                    }
                }
            }
        } else {
            throw new XMLParseException();
        }

        if ((group == null) || (messageID == null) || (from == null) ||
            (header == null) || (body == null) || (date == null)) {
            throw new XMLParseException();
        }

        return new BasicNewsItem(group, messageID, from, header, body, date);
    }

    /**
     * Parses a group list XML document into a GroupIterator.
     *
     * @param xmlGroupList The group list XML document to parse.
     *
     * @return A GroupIterator of all the groups.
     *
     * @throws XMLParseException if the xmlGroupList document does not
     *         contain a valid XML group list.
     */
    public static GroupIterator parseGroupList(XmlNode xmlGroupList)
                                        throws XMLParseException {
        List list = new ArrayList();
        Enumeration elements = xmlGroupList.children();

        // confirm we are a group-list XML document
        if (xmlGroupList.getLocalName().equals("group-list")) {
            while (elements.hasMoreElements()) {
                XmlNode element = (XmlNode)elements.nextElement();

                String tag = element.getLocalName();

                // add all groups to our eventual iterator
                // ignore non-group tags.
                if (tag.equals("group")) {
                    String group = (String)element.getChildAt(0);

                    list.add(group);
                }
            }
        } else {
            throw new XMLParseException();
        }

        // build the iterator adapter
        return new BasicGroupIterator(list.iterator());
    }

    /**
     * Parses a new-news XML document into a MessageIDIterator.
     *
     * @param xmlNewNews The XML document to parse.
     *
     * @return The constructed MessageIDIterator.
     *
     * @throws XMLParseException if the XML document is not a valid new-news
     *         XML document.
     */
    public static MessageIDIterator parseNewNews(XmlNode xmlNewNews)
                                          throws XMLParseException {
        List list = new ArrayList();
        Enumeration elements = xmlNewNews.children();

        // confirm we are a new-news XML document
        if (xmlNewNews.getLocalName().equals("new-news")) {
            while (elements.hasMoreElements()) {
                XmlNode element = (XmlNode)elements.nextElement();

                String tag = element.getLocalName();

                // add all message IDs to our eventual iterator
                // ignore invalid tags.
                if (tag.equals("messageID")) {
                    String messageID = (String)element.getChildAt(0);

                    list.add(messageID);
                }
            }
        } else {
            throw new XMLParseException();
        }

        // build the iterator adapter
        return new BasicMessageIDIterator(list.iterator());
    }

    /**
     * Parses a group-ids XML document into a MessageIDIterator.
     *
     * @param xmlGroupIDs The XML document to parse.
     *
     * @return The constructed MessageIDIterator.
     *
     * @throws XMLParseException if the XML document is not a valid
     *         group-ids XML document.
     */
    public static MessageIDIterator parseGroupIDs(XmlNode xmlGroupIDs)
                                           throws XMLParseException {
        List list = new ArrayList();
        Enumeration elements = xmlGroupIDs.children();

        // confirm we are a new-news XML document
        if (xmlGroupIDs.getLocalName().equals("group-ids")) {
            while (elements.hasMoreElements()) {
                XmlNode element = (XmlNode)elements.nextElement();

                String tag = element.getLocalName();

                // add all message IDs to our eventual iterator
                // ignore invalid tags.
                if (tag.equals("id")) {
                    String messageID = (String)element.getChildAt(0);

                    list.add(messageID);
                }
            }
        } else {
            throw new XMLParseException();
        }

        // build the iterator adapter
        return new BasicMessageIDIterator(list.iterator());
    }
}
