/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datatypes;

import java.util.Date;

/**
 * Represents a container of newsgroups and behaviors for administering
 * messages on these groups.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.3, 03/28/03
 */
public interface GroupMap {
    /**
     * Queries if this container has a group with a particular name.
     *
     * @param groupName The name of the group for which to check.
     *
     * @return true if this group already exists; false otherwise.
     */
    public boolean containsGroup(String groupName);

    /**
     * Creates a new group.
     *
     * @param groupName The name of the group to create.
     *
     * @throws DuplicateGroupException if there is already a group by this
     *         name.
     */
    public void addGroup(String groupName) throws DuplicateGroupException;

    /**
     * Queries if this container has a message with a particular message ID.
     *
     * @param messageID The ID of the message to search for.
     *
     * @return true if the message is in this container; false otherwise.
     */
    public boolean containsMessage(String messageID);

    /**
     * Posts a message to one of the groups.
     *
     * @param message The NewsItem to post.
     *
     * @throws DuplicateMessageException if this message already exists.
     * @throws NoSuchGroupException if this message is for a non-existant
     *         group.
     */
    public void postMessage(NewsItem message)
                     throws DuplicateMessageException, NoSuchGroupException;

    /**
     * Gets the header of a message with a particular message ID.
     *
     * @param messageID The ID of the message for which to find the header.
     *
     * @return The message header.
     *
     * @throws NoSuchMessageException if no message exists with this message
     *         ID.
     */
    public String getHeader(String messageID) throws NoSuchMessageException;

    /**
     * Gets the message associated with a message ID.
     *
     * @param messageID The ID of the message to retrieve.
     *
     * @return The retrieved message.
     *
     * @throws NoSuchMessageException if no message exists with this message
     *         ID.
     */
    public NewsItem getMessage(String messageID)
                        throws NoSuchMessageException;

    /**
     * Gets all the messages since a particular date.
     *
     * @param sinceWhen The date to find messages since.
     *
     * @return A NewsItemIterator containing all the messages posted since
     *         the date.
     */
    public NewsItemIterator getMessages(Date sinceWhen);

    /**
     * Gets all the message IDs for a particular group.
     *
     * @param groupName The name of the group to retrieve message IDs for.
     *
     * @return A MessageIDIterator containing all the message IDs.
     *
     * @throws NoSuchGroupException if no such group exists.
     */
    public MessageIDIterator getMessages(String groupName)
                                  throws NoSuchGroupException;

    /**
     * Gets all the group names in this container.
     *
     * @return A GroupIterator of all the group names.
     */
    public GroupIterator getGroups();
}
