/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package driver;

import java.util.regex.Pattern;

import protocol.AbstractNntp;
import protocol.Nntp;
import protocol.ServerBindException;

import soaprmi.Remote;
import soaprmi.RemoteException;

import soaprmi.server.UnicastRemoteObject;

/**
 * The server program frontend to the web servers NNTP-like protocol.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.4 04/11/03
 */
public class Server {
    private Nntp server;
    private int port = DEFAULT_PORT;
    private String parentServer;

    /**
     * The default port to run the server on.
     */
    public static final int DEFAULT_PORT = 48526;

    /**
     * The program's name.
     */
    public static final String NAME_STRING = "DOS - The Design Oriented Server";

    /**
     * The current version string.
     */
    public static final String VERSION_STRING = "1.0.3";

    /**
     * The copyright information.
     */
    public static final String COPYRIGHT_STRING = "Copyright (C) 2003 John David Ratliff, Yohei Kaneko";

    /**
     * Creates a new Server object. Disallow foreign creation.
     */
    private Server() {
        // only allow the main method to build servers
    }

    /**
     * Displays the program's version string.
     */
    private void printVersion() {
        System.out.println(NAME_STRING + " version " + VERSION_STRING);
        System.out.println(COPYRIGHT_STRING);
    }

    /**
     * Display's program usage information.
     */
    private void printHelpMenu() {
        this.printVersion();

        System.out.println();
        System.out.println("usage: java Server [ args ]");
        System.out.println("  --port           The port to run the server on.");
        System.out.println("  --parent  or -p  The parent of this server, e.g. http://parent:port");
        System.out.println("  --help    or -h  Display this help screen.");
        System.out.println("  --version or -v  Display the version number.");
    }

    /**
     * Sets up the server options based on the command line arguments.
     *
     * @param args The command line arguments.
     */
    private void setArgs(String[] args) {
        for (int index = 0; index < args.length; index++) {
            String arg = args[index];

            // check for all argument possibilities
            if (arg.compareToIgnoreCase("--port") == 0) {
                // the server port argument
                if (++index < args.length) {
                    // make sure they supplied the argument
                    String port = args[index];

                    try {
                        int temp = Integer.parseInt(port);

                        if ((temp >= TCPConstants.MIN_PORT_NUMBER) &&
                            (temp <= TCPConstants.MAX_PORT_NUMBER)) {
                            this.port = temp;
                        } else {
                            System.err.println("Server: port must be between " +
                                               TCPConstants.MIN_PORT_NUMBER +
                                               " and " +
                                               TCPConstants.MAX_PORT_NUMBER +
                                               ".");
                        }
                    } catch (NumberFormatException e) {
                        System.err.println("Server: " + port +
                                           " is not a valid port number.");
                    }
                } else {
                    System.err.println("Server: no port specified!");
                }
            } else if ((arg.compareToIgnoreCase("--parent") == 0) ||
                       (arg.compareToIgnoreCase("-p") == 0)) {
                if (++index < args.length) {
                    String parent = args[index];

                    if (Pattern.matches("http://[a-zA-Z0-9]+((\\.[a-zA-Z0-9]+)*\\.[a-zA-Z0-9]+)?:\\d{1,5}",
                                        parent)) {
                        this.parentServer = parent;
                    } else {
                        System.out.println("Server: parent server must be of the form http://server:port, e.g. http://bobac.cs:24301");
                    }
                } else {
                    System.err.println("Server: no parent server specified!");
                }
            } else if ((arg.compareToIgnoreCase("--help") == 0) ||
                       (arg.compareToIgnoreCase("-h") == 0)) {
                // display the help menu and exit
                this.printHelpMenu();
                System.exit(-1);
            } else if ((arg.compareToIgnoreCase("--version") == 0) ||
                       (arg.compareTo("-v") == 0)) {
                // display the version information and exit
                this.printVersion();
                System.exit(-1);
            } else {
                // invalid option. print the help menu and exit
                this.printHelpMenu();
                System.exit(-1);
            }
        }

        // print out the server options
        System.out.println("----------------------------------------------------");
        System.out.println("Server: port = " + this.port);

        if (this.parentServer != null) {
            System.out.println("Server: parent server = " +
                               this.parentServer);
        } else {
            System.out.println("Server: this server is the root server.");
        }

        System.out.println("----------------------------------------------------");
        System.out.println();
    }

    /**
     * Starts and exports the Nntp service.
     */
    private void startServer() {
        try {
            // build the Nntp interface
            if (this.parentServer == null) {
                this.server = AbstractNntp.getInstance();
            } else {
                this.server = AbstractNntp.getInstance(this.parentServer);
            }

            System.out.println("Server: binding " +
                               ((this.parentServer == null) ? "root" : "child") +
                               " server on TCP port " + this.port);

            Remote service = UnicastRemoteObject.exportObject(this.server,
                                                              this.port);

            System.out.println("Server: successfully started Nntp service on port " +
                               this.port);
        } catch (ServerBindException e) {
            System.err.println("Server: unable to bind server...");
        } catch (RemoteException e) {
            System.err.println("Server: unable to export Nntp service on port " +
                               this.port);
        }
    }

    /**
     * The start method. Called by the java runtime to start this program.
     *
     * @param args The command line argumets. Use java --help for more
     *        information.
     */
    public static void main(String[] args) {
        Server server = new Server();

        // supply the server arguments
        server.setArgs(args);

        // start the server
        server.startServer();
    }
}
