/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package protocol;

import java.util.Date;

import datastructures.BasicGroupMap;
import datastructures.BasicXMLFactory;
import datastructures.XMLParseException;
import datastructures.XMLParser;

import datatypes.DuplicateMessageException;
import datatypes.GroupIterator;
import datatypes.GroupMap;
import datatypes.MessageIDIterator;
import datatypes.NewsItem;
import datatypes.NewsItemIterator;
import datatypes.NoSuchGroupException;
import datatypes.NoSuchMessageException;
import datatypes.XMLFactory;

import org.gjt.xpp.XmlNode;

import soaprmi.RemoteException;

/**
 * An implementation of the Nntp interface for a simple NNTP-like news
 * transfer protocol.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.8, 04/11/03
 */
class BasicNntp extends AbstractNntp implements Nntp {
    protected GroupMap groups = new BasicGroupMap();
    private XMLFactory xmlFactory = BasicXMLFactory.getInstance();

    /**
     * Creates a new BasicNntp object.
     */
    protected BasicNntp() {
        // disallow outside creation
    }

    /**
     * Initializes the root server groups.
     *
     * @throws ServerBindException if the initialization fails.
     */
    protected void initServer() throws ServerBindException {
        System.out.println("Server: initializing the root server...");
        System.out.println("Server: building 'seed' newsgroups...");

        // build the 'seed' groups
        // ac.csci.b538, ac.csci.c343, ac.csci.a348
        this.groups.addGroup("ac.csci.b538");
        this.groups.addGroup("ac.csci.c343");
        this.groups.addGroup("ac.csci.a348");

        System.out.println("Server: root server ready...");
    }

    /**
     * Verifies the alive-ness of the server.
     *
     * @throws RemoteException if the server is not running.
     */
    public void ping() throws RemoteException {
        // this method is only used to test for RemoteExceptions
        // if a client tries to 'ping' us, and gets a RemoteException
        // they can assume we are not running...
    }

    /**
     * Gets a list of all the known newsgroups.
     *
     * @return An XmlNode containing all the newsgroups this server knows
     *         about.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public XmlNode list() throws RemoteException {
        System.out.println("BasicNntp: client is requesting newsgroup list...");

        GroupIterator iterator = this.groups.getGroups();

        // the group-list tag
        String xmlString = "<group-list>";

        // append each group using a group tag
        while (iterator.hasMoreGroups()) {
            String group = iterator.nextGroup();

            xmlString += ("<group>" + group + "</group>");
        }

        xmlString += "</group-list>";

        // return the factory constructed XmlNode
        return this.xmlFactory.buildXmlNode(xmlString);
    }

    /**
     * Gets all the message IDs for a particular newsgroup.
     *
     * @param groupName The name of the group for which to retrieve message
     *        IDs.
     *
     * @return XmlNode containing the group message IDs.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public XmlNode groupIDs(String groupName) throws RemoteException {
        System.out.println("BasicNntp: client is requesting group IDs...");

        // the group ID tag
        String xmlString = "<group-ids>";

        try {
            MessageIDIterator iterator = this.groups.getMessages(groupName);

            // add all the message IDs
            while (iterator.hasMoreMessageIDs()) {
                String messageID = iterator.nextMessageID();

                xmlString += ("<id>" + messageID + "</id>");
            }
        } catch (NoSuchGroupException e) {
            System.err.println("Server: client asked for invalid group " +
                               groupName);
        }

        // close the group ID tag
        xmlString += "</group-ids>";

        // return the factory constructed XmlNode
        return this.xmlFactory.buildXmlNode(xmlString);
    }

    /**
     * Gets the message header string.
     *
     * @param messageID The ID of the message whose header to retrieve.
     *
     * @return The message header string.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public String head(String messageID) throws RemoteException {
        System.out.println("BasicNntp: client is requesting header for message " +
                           messageID);

        String header = "error: no such message";

        try {
            header = this.groups.getHeader(messageID);
        } catch (NoSuchMessageException e) {
            System.err.println("Server: client asked for an invalid message ID " +
                               messageID);
        }

        return header;
    }

    /**
     * Gets a news message.
     *
     * @param messageID The ID of the message to retrieve.
     *
     * @return An XmlNode containing the message.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public XmlNode getMessage(String messageID) throws RemoteException {
        System.out.println("BasicNntp: client is requesting message " +
                           messageID);

        try {
            NewsItem message = this.groups.getMessage(messageID);

            return message.toXmlNode();
        } catch (NoSuchMessageException e) {
            System.err.println("Server: client asked for an invalid message ID " +
                               messageID);
        }

        // if we got here, they requested an invalid messageID
        // this is bad. what should be returned here...
        // we will return an XML node that is an error message.
        // this should be checked by clients...
        String xmlString = "<error>invalid message ID</error>";

        return this.xmlFactory.buildXmlNode(xmlString);
    }

    /**
     * Posts a message to this server, so long as the server does not
     * already have said message.
     *
     * @param newsItem The XmlNode containing the news item being posted.
     *
     * @return true if this message was accepted; false otherwise.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public boolean iHave(XmlNode newsItem) throws RemoteException {
        System.out.println("BasicNntp: client is attempting to post a message...");

        NewsItem message = null;

        try {
            message = XMLParser.parseNewsItem(newsItem);
            message.setDate(new Date());

            this.groups.postMessage(message);
        } catch (XMLParseException e) {
            // invalid message
            System.err.println("Server: error parsing XmlNode into NewsItem!");

            return false;
        } catch (DuplicateMessageException e) {
            // duplicate message
            System.err.println("Server: ignoring duplicate message " +
                               message.getMessageID());

            return false;
        } catch (NoSuchGroupException e) {
            // posting to an invalid group
            System.err.println("Server: client attempted to post to invalid group " +
                               message.getGroup());

            return false;
        }

        return true;
    }

    /**
     * Gets all news this server has received since a certain date.
     *
     * @param dateLastChecked The date the last query was performed.
     *
     * @return An XmlNode containing all the new news message IDs received
     *         since the date last checked.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public XmlNode newNews(long dateLastChecked) throws RemoteException {
        Date sinceWhen = new Date(dateLastChecked);

        System.out.println("BasicNntp: client is requesting new news since " +
                           sinceWhen);

        String xmlString = "<new-news>";
        NewsItemIterator iterator = this.groups.getMessages(sinceWhen);

        // add all message to the XML document
        while (iterator.hasMoreNewsItems()) {
            NewsItem message = iterator.nextNewsItem();
            String messageID = message.getMessageID();

            xmlString += ("<messageID>" + messageID + "</messageID>");
        }

        // close the new news tag
        xmlString += "</new-news>";

        return this.xmlFactory.buildXmlNode(xmlString);
    }
}
