/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package protocol;

import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import datastructures.XMLParseException;
import datastructures.XMLParser;

import datatypes.GroupIterator;
import datatypes.MessageIDIterator;
import datatypes.NewsItem;

import org.gjt.xpp.XmlNode;

import soaprmi.RemoteException;

import soaprmi.soaprpc.SoapServices;

/**
 * An implementation of the Nntp web service for child servers in a tree
 * hierarchy.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.5, 04/11/03
 */
class ChildNntp extends BasicNntp implements Nntp {
    private Nntp parent;
    private Timer requestTimer;
    private long lastChecked = 0;

    /**
     * How often to wait between update requests.
     */
    public static final int DEFAULT_REQUEST_INTERVAL = 10000;

    /**
     * Creates a new ChildNntp object.
     *
     * @param parentServer The parent's web service address.
     *
     * @throws ServerBindException if the parent pointer cannot be
     *         established.
     */
    protected ChildNntp(String parentServer) throws ServerBindException {
        System.out.println("Child: connecting to parent server " +
                           parentServer);

        try {
            // get the parent service interface
            SoapServices soap = SoapServices.getDefault();
            this.parent = (Nntp)soap.createStartpoint(parentServer,
                                                      new Class[] {Nntp.class});
        } catch (RemoteException e) {
            throw new ServerBindException();
        }

        System.out.println("Child: parent pointer established...");
    }

    /**
     * Initializes this child server.
     *
     * @param parentServer The parent's web service address.
     *
     * @throws ServerBindException if the server initialization fails.
     */
    protected void initServer(String parentServer)
                       throws ServerBindException {
        System.out.println("Child: requesting group list from parent...");

        try {
            // build the group list from the parent
            XmlNode xmlGroupList = this.parent.list();

            GroupIterator iterator = XMLParser.parseGroupList(xmlGroupList);

            while (iterator.hasMoreGroups()) {
                String group = iterator.nextGroup();

                System.out.println("Child: adding group " + group);

                this.groups.addGroup(group);
            }
        } catch (RemoteException e) {
            System.err.println("ChildNntp: child was unable to retrieve newsgroup list from parent.");
            throw new ServerBindException();
        } catch (XMLParseException e) {
            // this shouldn't really happen
            // if it does, it's probably a bug
            e.printStackTrace();
            throw new ServerBindException();
        }

        System.out.println("Child: child server ready...");

        // start the request timer
        this.requestTimer = new Timer();
        this.requestTimer.scheduleAtFixedRate(new RequestTimer(), 0,
                                              DEFAULT_REQUEST_INTERVAL);
    }

    /**
     * Posts a message to this server, so long as the server does not
     * already have said message.
     *
     * @param newsItem The XmlNode containing the news item being posted.
     *
     * @return true if this message was accepted; false otherwise.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public boolean iHave(XmlNode newsItem) throws RemoteException {
        System.out.println("ChildNntp: propogating message post...");

        // propogate the message up the hierarchy
        boolean success = this.parent.iHave(newsItem);

        // add the message to the local repository
        super.iHave(newsItem);

        // return the propogation success
        return success;
    }

    /**
     * Checks for new messages once per DEFAULT_REQUEST_INTERVAL.
     */
    private void requestNewNews() {
        // retrieve new messages from parent
        try {
            System.out.println("ChildNntp: requesting new messages from parent...");

            // mark the current time
            long beforeTime = System.currentTimeMillis();

            // get the new news and parse it into an iterator
            XmlNode xmlNewNews = this.parent.newNews(this.lastChecked);
            MessageIDIterator iterator = XMLParser.parseNewNews(xmlNewNews);

            // add all the new messages
            while (iterator.hasMoreMessageIDs()) {
                // get the next message and parse it into a NewsItem
                String messageID = iterator.nextMessageID();

                System.out.println("ChildNntp: attempting to add new message " +
                                   messageID);

                // confirm we didn't add this message already...
                if (!this.groups.containsMessage(messageID)) {
                    XmlNode xmlNewsItem = this.parent.getMessage(messageID);
                    NewsItem message = XMLParser.parseNewsItem(xmlNewsItem);
                    message.setDate(new Date());

                    // add the message to the local repository
                    this.groups.postMessage(message);
                }
            }

            // update the time last checked
            this.lastChecked = beforeTime;

            System.out.println("ChildNntp: successfully retrieved and updated message cache from parent...");
        } catch (XMLParseException e) {
            // this should really never happen
            e.printStackTrace();
        } catch (RemoteException e) {
            // this should only happen if a network layer
            // error occurs. i.e., rarely.
            e.printStackTrace();
        }
    }

    /**
     * The task called by the timer to request a news update.
     *
     * @author John David Ratliff
     * @author Yohei Kaneko
     * @version 1.1, 04/03/03
     */
    private class RequestTimer extends TimerTask {
        /**
         * The timer's action method.
         */
        public void run() {
            requestNewNews();
        }
    }
}
