/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package protocol;

import org.gjt.xpp.XmlNode;

import soaprmi.Remote;
import soaprmi.RemoteException;

/**
 * An interface specifying the news server protocol.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.2, 04/11/03
 */
public interface Nntp extends Remote {
    /**
     * Verifies the alive-ness of the server.
     *
     * @throws RemoteException if the server is not running.
     */
    public void ping() throws RemoteException;

    /**
     * Gets a list of all the known newsgroups.
     *
     * @return An XmlNode containing all the newsgroups this server knows
     *         about.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public XmlNode list() throws RemoteException;

    /**
     * Gets all the message IDs for a particular newsgroup.
     *
     * @param groupName The name of the group for which to retrieve message
     *        IDs.
     *
     * @return XmlNode containing the group message IDs.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public XmlNode groupIDs(String groupName) throws RemoteException;

    /**
     * Gets the message header string.
     *
     * @param messageID The ID of the message whose header to retrieve.
     *
     * @return The message header string.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public String head(String messageID) throws RemoteException;

    /**
     * Gets a news message.
     *
     * @param messageID The ID of the message to retrieve.
     *
     * @return An XmlNode containing the message.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public XmlNode getMessage(String messageID) throws RemoteException;

    /**
     * Posts a message to this server, so long as the server does not
     * already have said message.
     *
     * @param newsItem The XmlNode containing the news item being posted.
     *
     * @return true if this message was accepted; false otherwise.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public boolean iHave(XmlNode newsItem) throws RemoteException;

    /**
     * Gets all news this server has received since a certain date.
     *
     * @param dateLastChecked The date the last query was performed.
     *
     * @return An XmlNode containing all the new news message IDs received
     *         since the date last checked.
     *
     * @throws RemoteException if something goes wrong in the network layer.
     */
    public XmlNode newNews(long dateLastChecked) throws RemoteException;
}
