/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package protocol;

import java.util.Date;

import datastructures.BasicNewsItem;
import datastructures.XMLParseException;
import datastructures.XMLParser;

import datatypes.GroupIterator;
import datatypes.MessageIDIterator;

import org.gjt.xpp.XmlNode;

import soaprmi.RemoteException;

import soaprmi.server.UnicastRemoteObject;

/**
 * Unit Tests for the package protocol.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.1, 04/04/03
 */
public class UnitTests {
    /**
     * Tests the protocol.BasicNntp class.
     */
    public static void testBasicNntp() {
        try {
            // create root server and export the service
            Nntp root = AbstractNntp.getInstance();
            UnicastRemoteObject.exportObject(root, 48526);

            // create the child service
            Nntp child = AbstractNntp.getInstance("http://127.0.0.1:48526");

            // default server groups
            // if these groups change, these tests will FAIL!
            String group = "ac.csci.b538";
            String group2 = "ac.csci.c343";
            String group3 = "ac.csci.a348";

            // create some test messages
            String messageID;

            // create some test messages
            String messageID2;

            // create some test messages
            String messageID3;
            Date date;
            Date date2;
            Date date3;
            BasicNewsItem message;
            BasicNewsItem message2;
            BasicNewsItem message3;

            messageID = "101.yohei@localhost";
            date = new Date(100);
            message = new BasicNewsItem(group, messageID, "yohei@yohei.com",
                                        "header 1", "message body 1", date);

            messageID2 = "102.dom@localhost";
            date2 = new Date(200);
            message2 = new BasicNewsItem(group, messageID2, "dom@yohei.com",
                                         "header 2", "message body 2", date2);

            messageID3 = "103.gannon@whale.cs.indiana.edu";
            date3 = new Date(1000);
            message3 = new BasicNewsItem(group3, messageID3,
                                         "gannon@cs.indiana.edu",
                                         "header 3", "message body 3", date3);

            // test group list of children (Nntp.list)
            XmlNode childGroups = child.list();

            GroupIterator iterator = XMLParser.parseGroupList(childGroups);
            String nextGroup;

            // should be 3 groups in the iterator
            // ac.csci.b538, ac.csci.a348, ac.csci.c343
            assert (iterator.hasMoreGroups());

            nextGroup = iterator.nextGroup();

            assert (nextGroup.equals(group) || nextGroup.equals(group2) ||
            nextGroup.equals(group3));

            nextGroup = iterator.nextGroup();

            assert (nextGroup.equals(group) || nextGroup.equals(group2) ||
            nextGroup.equals(group3));

            nextGroup = iterator.nextGroup();

            assert (nextGroup.equals(group) || nextGroup.equals(group2) ||
            nextGroup.equals(group3));

            assert (!iterator.hasMoreGroups());

            // test message posting (Nntp.iHave)
            // pull-down messages
            root.iHave(message.toXmlNode());
            root.iHave(message2.toXmlNode());

            // propogation messages
            child.iHave(message3.toXmlNode());

            // test group message (Nntp.groupIDs)
            XmlNode group3IDs = child.groupIDs(group3);
            MessageIDIterator iterator2 = XMLParser.parseGroupIDs(group3IDs);

            assert (iterator2.hasMoreMessageIDs());

            String nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID3));

            assert (!iterator2.hasMoreMessageIDs());

            // wait until the child pulls down the server messages
            try {
                Thread.sleep(ChildNntp.DEFAULT_REQUEST_INTERVAL);
            } catch (InterruptedException e) {
                // this will rarely (if ever) happen
                e.printStackTrace();
            }

            XmlNode group1IDs = child.groupIDs(group);
            iterator2 = XMLParser.parseGroupIDs(group1IDs);

            assert (iterator2.hasMoreMessageIDs());

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2));

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2));

            assert (!iterator2.hasMoreMessageIDs());

            // test that headers retrieve successfully (Nntp.head)
            String header = child.head(messageID);

            assert (header.equals(message.getHeader()));

            // test that messages retrieve successfully (Nntp.getMessage)
            XmlNode msg = child.getMessage(messageID2);
            BasicNewsItem newsItem = (BasicNewsItem)XMLParser.parseNewsItem(msg);

            assert (newsItem.equals(message2));

            // test new messages since date (Nntp.newNews)
            // get new news since the epoch (should == 3 messages)
            XmlNode xmlNewIDs = child.newNews(0);
            iterator2 = XMLParser.parseNewNews(xmlNewIDs);

            assert (iterator2.hasMoreMessageIDs());

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2) ||
            nextID.equals(messageID3));

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2) ||
            nextID.equals(messageID3));

            nextID = iterator2.nextMessageID();

            assert (nextID.equals(messageID) || nextID.equals(messageID2) ||
            nextID.equals(messageID3));

            assert (!iterator2.hasMoreMessageIDs());
        } catch (XMLParseException e) {
            // this should not happen, if things are working proper
            assert false;
        } catch (RemoteException e) {
            // unable to bind the server.
            assert false;
        } catch (ServerBindException e) {
            // unable to create the server
            assert false;
        }
    }

    /**
     * Runs the unit tests for the package protocol. You must enable
     * compile-time assertion support in the Java 1.4 compiler and run-time
     * assertion enabling in the java runtime. To do this, use javac
     * -source 1.4 protocol.UnitTests to compile and java -ea
     * protocol.UnitTests to run.
     *
     * @param args Not used.
     */
    public static void main(String[] args) {
        // test the BasicNntp class
        testBasicNntp();

        // kill the running servers
        // would be nice if there was a cleaner way to do this...
        System.exit(0);
    }
}
