/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.net.InetAddress;
import java.net.UnknownHostException;

import java.util.Date;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import javax.swing.border.EtchedBorder;

import datastructures.BasicNewsItem;

import datatypes.NewsItem;
import datatypes.UserSettings;

import org.gjt.xpp.XmlNode;

import protocol.Nntp;

import soaprmi.RemoteException;
import driver.Debug;

/**
 * A dialog for composing messages to post on news servers. GUI elements
 * were adapted from code produced by NetBeans 3.4.1.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.4, 04/11/03
 */
public class ComposeDialog extends JDialog {
    private JLabel messageID;
    private JTextArea body;
    private JTextField group;
    private JTextField header;
    private JTextField from;
    private UserSettings settings;
    private Nntp server;
    private boolean valid = false;
    private static long unique = (new Date()).getTime();
    private static int count = 100;
    private static String hostname;

    static {
        try {
            hostname = InetAddress.getLocalHost().getHostName();
        } catch (UnknownHostException e) {
            // I can't see this happening
            // but if it fails, generate a random hostname
            hostname = "host" + (int)(Math.random() * 1000);
        }
    }

    /**
     * Creates a new ComposeDialog object.
     *
     * @param server The server to post to.
     * @param settings The user settings.
     * @param group The group to post onto.
     * @param parent The parent frame.
     * @param modal If the dialog should be modal.
     */
    public ComposeDialog(Nntp server, UserSettings settings, String group,
                         Frame parent, boolean modal) {
        super(parent, modal);
        this.server = server;
        this.settings = settings;
        initComponents(group);
    }

    /**
     * Generate a 'unique' message ID.
     *
     * @return The message ID.
     */
    private String generateMessageID() {
        count++;
        return (count + "." + settings.getUserName() + "@" + hostname +
               " [" + unique + "]");
    }

    /**
     * Generates a from string from a username and email address.
     *
     * @return The built from string.
     */
    private String getFromString() {
        return (this.settings.getUserName() + " [" +
               this.settings.getUserEmail() + "]");
    }

    /**
     * Initializes the swing components.
     *
     * @param group The group you are posting to.
     */
    private void initComponents(String group) {
	String signature = "";

        // make sure group's been selected
        if (group == null) {
            group = "";
        }

	if (this.settings.getUserSignature().length() > 0) {
	    signature = "\n\n---\n" + this.settings.getUserSignature();
	}

        GridBagConstraints gridBagConstraints;

        // create the Swing components
        this.group = new JTextField(group);
        this.from = new JTextField(this.getFromString());
        this.header = new JTextField();
        this.messageID = new JLabel(this.generateMessageID());
        this.body = new JTextArea(signature);

        JPanel buttonPanel = new JPanel();
        JButton sendButton = new JButton();
        JButton cancelButton = new JButton();
        JSeparator separator = new JSeparator();
        JLabel groupLabel = new JLabel();
        JLabel fromLabel = new JLabel();
        JLabel headerLabel = new JLabel();
        JLabel messageIDLabel = new JLabel();
        JScrollPane bodyScroller = new JScrollPane();

        // set the layout manager
        this.getContentPane().setLayout(new GridBagLayout());

        // reclaim this on close
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setTitle("Compose Dialog");
        this.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent evt) {
                    closeDialog();
                }
            });

        buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        sendButton.setMnemonic('s');
        sendButton.setText("Send");
        sendButton.setToolTipText("Sends the message to the server");
        buttonPanel.add(sendButton);

        sendButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    valid = true;
                    validateAndClose();
                }
            });

        cancelButton.setMnemonic('c');
        cancelButton.setText("Cancel");
        cancelButton.setToolTipText("Cancels this message");
        buttonPanel.add(cancelButton);

        cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    valid = false;
                    closeDialog();
                }
            });

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        this.getContentPane().add(buttonPanel, gridBagConstraints);

        separator.setForeground(new Color(51, 51, 51));
        separator.setPreferredSize(new Dimension(0, 1));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(0, 5, 15, 5);
        this.getContentPane().add(separator, gridBagConstraints);

        groupLabel.setText("Newsgroup: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        this.getContentPane().add(groupLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        this.getContentPane().add(this.group, gridBagConstraints);

        fromLabel.setText("From: ");
        fromLabel.setToolTipText("null");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        this.getContentPane().add(fromLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        this.getContentPane().add(this.from, gridBagConstraints);

        headerLabel.setText("Subject: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        this.getContentPane().add(headerLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        this.getContentPane().add(this.header, gridBagConstraints);

        messageIDLabel.setText("Identifier: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 5;
        gridBagConstraints.insets = new Insets(0, 5, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        this.getContentPane().add(messageIDLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        this.getContentPane().add(this.messageID, gridBagConstraints);

        bodyScroller.setPreferredSize(new Dimension(700, 300));
        this.body.setLineWrap(true);
        this.body.setBorder(new EtchedBorder(EtchedBorder.RAISED,
                                             Color.white, Color.black));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 6;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 5, 5, 5);
        gridBagConstraints.weighty = 1.0;
        bodyScroller.setViewportView(this.body);
        this.getContentPane().add(bodyScroller, gridBagConstraints);

        this.pack();
    }

    /**
     * Validates the message, posts it, and closes this dialog.
     */
    private void validateAndClose() {
        Frame frame = (Frame)this.getParent();

        if (this.group.getText().length() == 0) {
            // empty group
            this.valid = false;

            JOptionPane.showMessageDialog(frame, "Group cannot be empty!",
                                          "Invalid Newsgroup",
                                          JOptionPane.ERROR_MESSAGE);
        } else if (this.from.getText().length() == 0) {
            // empty from
            this.valid = false;

            JOptionPane.showMessageDialog(frame,
                                          "From field cannot be empty!",
                                          "Invalid From address.",
                                          JOptionPane.ERROR_MESSAGE);
        } else if (this.header.getText().length() == 0) {
            // empty header
            this.valid = false;

            JOptionPane.showMessageDialog(frame, "Subject cannot be empty!",
                                          "Invalid Subject",
                                          JOptionPane.ERROR_MESSAGE);
        } else if (this.body.getText().length() == 0) {
            // empty body
            this.valid = false;

            JOptionPane.showMessageDialog(frame, "Body cannot be empty!",
                                          "Invalid Body",
                                          JOptionPane.ERROR_MESSAGE);
        }

        if (this.isPropagable()) {
            XmlNode newsItem = this.getMessage().toXmlNode();

            try {
                if (!server.iHave(newsItem)) {
                    JOptionPane.showMessageDialog(frame,
                                                  "Unable to post message!",
                                                  "Server Error",
                                                  JOptionPane.ERROR_MESSAGE);
                }
            } catch (RemoteException e) {
                JOptionPane.showMessageDialog(frame,
                                              "Unable to post message!",
                                              "Server Error",
                                              JOptionPane.ERROR_MESSAGE);
            }
            
            if (Debug.DEBUG_MODE) {
                System.out.println("ComposeDialog: sending message " + this.messageID.getText());
                System.out.println("ComposeDialog: at time " + new Date());
            }

            // close the compose dialog
            closeDialog();
        }
    }

    /**
     * Gets a NewsItem from the compose fields.
     *
     * @return The constructed NewsItem.
     *
     * @throws NonPropagableException if the message is not sendable.
     */
    private NewsItem getMessage() throws NonPropagableException {
        if (!this.isPropagable()) {
            throw new NonPropagableException();
        }

        String group = this.group.getText();
        String messageID = this.messageID.getText();
        String from = this.from.getText();
        String header = this.header.getText();
        String body = this.body.getText();
        Date date = new Date();

        return new BasicNewsItem(group, messageID, from, header, body, date);
    }

    /**
     * Queries propagability.
     *
     * @return true if the message is valid and sendable; false otherwise.
     */
    private boolean isPropagable() {
        return this.valid;
    }

    /**
     * Closes this dialog.
     */
    private void closeDialog() {
        setVisible(false);
        dispose();
    }
}
